@@ -6,6 +6,7 @@ import os
 import zipfile
 import numpy as np
 import datetime
+from itertools import product
 
 
 class PreprocessDatasets:
@@ -181,30 +182,51 @@ class PreprocessDatasets:
         groups = generate_groups_data_matrix(groups)
         return groups
 
-    def _police(self):
-        path = self._get_dataset()
+    def _police(self, start_date='2021-01-01', end_date='2021-11-30'):
+        path = self._get_dataset(file_type='xlsx')
         if not path:
             return {}
         police = pd.read_excel(path)
+        cols = ['Crime', 'Beat', 'Street', 'ZIP']
+        cols_date = cols.copy()
+        cols_date.append('Date')
 
+        # Drop unwanted columns
         police = police.drop(['RMSOccurrenceHour', 'StreetName', 'Suffix', 'NIBRSDescription', 'Premise'], axis=1)
         police.columns = ['Id', 'Date', 'Crime', 'Count', 'Beat', 'Block', 'Street', 'City', 'ZIP']
         police = police.drop(['Id'], axis=1)
+        police = police.astype({'ZIP': 'string'})
 
         # Filter top 1000 series
-        police_top = police.groupby(['Crime', 'Beat', 'Street', 'ZIP']).sum().sort_values(by='Count', ascending=False).head(
-            self.top)
+        police_top = police.groupby(cols).sum().sort_values(by='Count', ascending=False).head(
+            self.top).drop('Count', axis=1).reset_index()
+        police = police.groupby(cols_date).sum().reset_index()
 
         # create a column marking df2 values
         police_top['marker'] = 1
 
         # join the two, keeping all of df1's indices
-        joined = pd.merge(police, police_top, on=['Crime', 'Beat', 'Street', 'ZIP'], how='left')
+        joined = pd.merge(police, police_top, on=cols, how='left')
         police_f = joined[joined['marker'] == 1][police.columns]
-
         police_f = police_f.reset_index().drop('index', axis=1)
-        police_f = police_f.groupby(['Date', 'Crime', 'Beat', 'Street', 'ZIP']).sum().reset_index().set_index('Date')
-        police_pivot = police_f.reset_index().pivot(index='Date', columns=['Crime', 'Beat', 'Street', 'ZIP'], values='Count')
+        police_f = police_f.groupby(cols_date).sum().reset_index().set_index('Date')
+
+        # build a reference dataframe with all the dates to be merges, as the original does not have data for all days
+        police_to_merge = police_f.reset_index().drop(['Date', 'Count'], axis=1).drop_duplicates()
+        idx = pd.date_range(start_date, end_date)
+        print(idx)
+        lens = len(idx)
+        rest_to_concat = pd.DataFrame(np.array([np.repeat(police_to_merge.iloc[:, i].values, lens) for i in range(len(cols)-1)]).T)
+        complete_data = pd.DataFrame(product(list(police_to_merge.iloc[:, -1]), idx))
+        frames = [rest_to_concat, complete_data]
+        police_base = pd.concat(frames, axis=1)
+        police_base.columns = cols_date
+        print(police_base)
+
+        police_f = police_f.reset_index()
+        police = police_base.merge(police_f, how='left', on=cols_date)
+
+        police_pivot = police.reset_index().pivot(index='Date', columns=cols, values='Count')
         police_pivot = police_pivot.fillna(0)
 
         groups_input = {
